#include "SoftwareCaster.h"

SoftwareCaster::SoftwareCaster(Environment & envEnvironment): m_envEnvironment(envEnvironment) {
	
	m_iResolution = HIGHRES;
	int lowRes = LOWRES;
	m_bResState = true;
	
	//Generating high res texture
	glGenTextures( 1, &m_uHighResTexture);
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, m_uHighResTexture );
    
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,m_iResolution,m_iResolution,0,GL_RGBA,GL_UNSIGNED_BYTE,NULL);
	
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable( GL_TEXTURE_2D);
	
	
	//Generating low res texture
	glGenTextures( 1, &m_uLowResTexture);
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, m_uLowResTexture );
    
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,lowRes,lowRes,0,GL_RGBA,GL_UNSIGNED_BYTE,NULL);
	
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable( GL_TEXTURE_2D);
    
};

SoftwareCaster::~SoftwareCaster(){};
void SoftwareCaster::idle(){};
void SoftwareCaster::underlay(){};

void SoftwareCaster::display(){
	bind();
};

void SoftwareCaster::overlay(){};
void SoftwareCaster::reshape(const unsigned int uWidth, const unsigned int uHeight){};

//changing to low resolution when changing the view distance or view direction
void SoftwareCaster::mousePress(const MouseEvent & mouEvent){
	m_bResState = false;
	
	m_iResolution  = LOWRES;
};

//changing back to high resolution
void SoftwareCaster::mouseRelease(const MouseEvent & mouEvent){
	m_bResState = true;
	m_iResolution = HIGHRES;
};
void SoftwareCaster::mouseMove(const MouseEvent & mouEvent){};
void SoftwareCaster::keyboardPress(const KeyboardEvent & keyEvent){};	
void SoftwareCaster::keyboardRelease(const KeyboardEvent & keyEvent){};

//binds the texture to the plane
void SoftwareCaster::bind() {
	
    int width, height;
    BYTE * data;

    width = m_iResolution;
    height =m_iResolution;
	data = new BYTE [width * height * 4 ];

	generateData(data);

    glPushMatrix();
	glPushAttrib(GL_ALL_ATTRIB_BITS);

	
	glEnable( GL_TEXTURE_2D );

	if (m_bResState) {
		glBindTexture( GL_TEXTURE_2D, m_uHighResTexture );
		glTexSubImage2D(GL_TEXTURE_2D,0,0,0,width,height,GL_RGBA,GL_UNSIGNED_BYTE,data);
	}
	else {
		glBindTexture( GL_TEXTURE_2D, m_uLowResTexture );
		glTexSubImage2D(GL_TEXTURE_2D,0,0,0,width,height,GL_RGBA,GL_UNSIGNED_BYTE,data);
	}
	glEnable(GL_BLEND);
	glBegin( GL_QUADS );
	glTexCoord2d(0.0,0.0); glVertex2d(-1,-1);
	glTexCoord2d(1.0,0.0); glVertex2d(1,-1);
	glTexCoord2d(1.0,1.0); glVertex2d(1,1);
	glTexCoord2d(0.0,1.0); glVertex2d(-1,1);
	glEnd();
	
	
	glDisable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable( GL_TEXTURE_2D);
	
	
	glPopAttrib();
	glPopMatrix();
	
	
    delete data ;

    

};

//generates a texture represented by an array of BYTE
void SoftwareCaster::generateData(BYTE * data) {
	int width = m_iResolution;
	int height = m_iResolution;

	float xPos = -1;
	float yPos = -1;
	float stepWidth = 2.0f/(float)width;
	float stepHeight = 2.0f/(float)height;

	ColorTransferFunction ctf = m_envEnvironment.GetBackgroundSelectionColorTransferFunction();
	LightingTransferFunction ltf = m_envEnvironment.GetBackgroundLightingTransferFunction();
	
	const Matrix matTransformation = (m_envEnvironment.GetProjectionTransformation() * (m_envEnvironment.GetViewingTransformation() * m_envEnvironment.GetDataTransformation())).GetInverse();
	const Matrix matInverseTransformation = matTransformation.GetInverse();

	const Vector vecView = (m_envEnvironment.GetViewingTransformation().GetInverse() * Vector(0.0,0.0,1.0f)).GetNormalized();
	const Vector vecLight = (m_envEnvironment.GetViewingTransformation().GetInverse() * Vector(-0.25,0.25,0.75f)).GetNormalized();
	const Vector vecHalf = (vecLight + vecView).GetNormalized();
	
	bool cutaway = m_envEnvironment.GetCutAwayEnabled();
	float threshold = 0.0;
	if (cutaway) {
		threshold = m_envEnvironment.GetCutAwayThreshold();
	}
	
	//Raycasting with early ray termination
	for (int y = 0; y < height; y++) {
		for (int x = 0; x< width; x++) {
		
			Vector vecStart(xPos, yPos,	-1.0f);
			Vector vecEnd(xPos, yPos, 1.0f);
			
			Vector vecRayStart = matTransformation * vecStart;
			Vector vecRayEnd = matTransformation * vecEnd;
			Vector vecRayDirection = vecRayEnd-vecRayStart;
			float fRayLength = vecRayDirection.GetMagnitude();

			vecRayDirection.normalize();
			
			Color C = Color(0,0,0,0);
			Vector vecRayColor = Vector(0.0,0.0,0.0);
			float rayColorAlpha = 0.0;
			bool terminate = false;
			for (Volume<DataVoxel>::RayIterator i(m_envEnvironment.GetDataVolume(),vecRayStart,vecRayDirection);!i.IsAtEnd();++i)
			{	

				float intensity= (*i).GetValue();
				unsigned int value =(int)(intensity * 511);
				
				unsigned int t = 0;
				
				Color c = ctf.Get(value,t);
				if (c.GetAlpha() >0) {
					
					Vector color = Vector(c.GetNormalizedRed(),c.GetNormalizedGreen(), c.GetNormalizedBlue());
					float colorAlpha = c.GetNormalizedAlpha();
					if (cutaway && intensity > threshold - 0.1 && intensity < threshold +0.1) {
						vecRayColor = Vector(0.0,0.0,0.0);
						rayColorAlpha = 0.0;
						colorAlpha = 1.0;
						terminate = true;
					}

					Vector gradient = (*i).GetGradient().GetNormalized();
					float length = (*i).GetGradient().GetMagnitude();
					
					Vector vecProducts = Vector(gradient.GetDot(vecLight),gradient.GetDot(vecHalf), length);
					

					//calculate lighting
					float ltfX = (vecProducts.GetX() + 1)*0.5f;
					float ltfY = (vecProducts.GetY() + 1)*0.5f;

					//the color transfer function and the lighting transfer function are boh 512*512
					ltfX*=511;
					ltfY*=511;

					Color lighting = ltf.Get((int)ltfX,(int)ltfY);
					Vector vecLight = Vector(lighting.GetNormalizedRed(), lighting.GetNormalizedGreen(), lighting.GetNormalizedBlue());
					float lightAlpha = lighting.GetNormalizedAlpha();
					lightAlpha = std::min(1.0f,lightAlpha+(1.0f-vecProducts.GetZ()));
					
					//calculate shading
					Vector vecShaded = color;
					float shadedAlpha = colorAlpha;
					vecShaded = (color * vecLight.GetY());
					vecShaded.SetX(vecShaded.GetX() +(vecLight.GetX() + vecLight.GetZ()));
					vecShaded.SetY(vecShaded.GetY() +(vecLight.GetX() + vecLight.GetZ()));
					vecShaded.SetZ(vecShaded.GetZ() +(vecLight.GetX() + vecLight.GetZ()));
					if (lightAlpha > 0.0) {shadedAlpha /= lightAlpha;}

					
					//accumulate color and alpha
					shadedAlpha *= (1.0f - rayColorAlpha);
					vecShaded *= shadedAlpha;
					vecRayColor += vecShaded;
					rayColorAlpha += shadedAlpha;

					
					if (rayColorAlpha > 0.8 && !cutaway) {
						terminate = true;
					}
					//early ray termination
					if (terminate) {break;}
				}
			}
			
			//prevent overshooting colors
			vecRayColor = Vector(std::min(1.0f,vecRayColor.GetX()),std::min(1.0f,vecRayColor.GetY()),std::min(1.0f,vecRayColor.GetZ()));
			
			vecRayColor *= 255;
			rayColorAlpha *=255;
			
			data[(x+y*width) * 4]   = (BYTE)vecRayColor.GetX();
			data[(x+y*width) * 4+1] = (BYTE)vecRayColor.GetY();
			data[(x+y*width) * 4+2] = (BYTE)vecRayColor.GetZ();
			data[(x+y*width) * 4+3] = (BYTE)rayColorAlpha;
				
			xPos += stepWidth;
		}	
		xPos=-1;
		yPos+=stepHeight;		
	}
};

